from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_api.logging.api_logger import ApiLogger
from bot_api.route.route import Route
from bot_core.service.config_service import ConfigService
from bot_data.abc.server_config_repository_abc import ServerConfigRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.server_afk_channel_ids_config import ServerAFKChannelIdsConfig
from bot_data.model.server_config import ServerConfig
from bot_data.model.server_team_role_ids_config import ServerTeamRoleIdsConfig
from bot_data.model.team_member_type_enum import TeamMemberTypeEnum
from bot_data.model.user_role_enum import UserRoleEnum
from bot_graphql.abc.query_abc import QueryABC
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class ServerConfigMutation(QueryABC):
    def __init__(
        self,
        logger: ApiLogger,
        bot: DiscordBotServiceABC,
        servers: ServerRepositoryABC,
        server_configs: ServerConfigRepositoryABC,
        db: DatabaseContextABC,
        config_service: ConfigService,
        permissions: PermissionServiceABC,
    ):
        QueryABC.__init__(self, "ServerConfigMutation")

        self._logger = logger
        self._bot = bot
        self._servers = servers
        self._server_configs = server_configs
        self._db = db
        self._config_service = config_service
        self._permissions = permissions

        self.set_field("updateServerConfig", self.resolve_update_server_config)

    def resolve_update_server_config(self, *_, input: dict):
        if "id" not in input:
            raise ValueError("Id not set")

        server_config = self._server_configs.get_server_config_by_id(int(input["id"]))
        self._can_user_mutate_data(Route.get_user().users[0].server, UserRoleEnum.technician)

        server_config.message_delete_timer = (
            input["messageDeleteTimer"] if "messageDeleteTimer" in input else server_config.message_delete_timer
        )
        server_config.notification_chat_id = (
            input["notificationChatId"] if "notificationChatId" in input else server_config.notification_chat_id
        )
        server_config.max_voice_state_hours = (
            input["maxVoiceStateHours"] if "maxVoiceStateHours" in input else server_config.max_voice_state_hours
        )
        server_config.xp_per_message = (
            input["xpPerMessage"] if "xpPerMessage" in input else server_config.xp_per_message
        )
        server_config.xp_per_reaction = (
            input["xpPerReaction"] if "xpPerReaction" in input else server_config.xp_per_reaction
        )
        server_config.max_message_xp_per_hour = (
            input["maxMessageXpPerHour"] if "maxMessageXpPerHour" in input else server_config.max_message_xp_per_hour
        )
        server_config.xp_per_ontime_hour = (
            input["xpPerOntimeHour"] if "xpPerOntimeHour" in input else server_config.xp_per_ontime_hour
        )
        server_config.xp_per_event_participation = (
            input["xpPerEventParticipation"]
            if "xpPerEventParticipation" in input
            else server_config.xp_per_event_participation
        )
        server_config.xp_per_achievement = (
            input["xpPerAchievement"] if "xpPerAchievement" in input else server_config.xp_per_achievement
        )
        server_config.afk_command_channel_id = (
            input["afkCommandChannelId"] if "afkCommandChannelId" in input else server_config.afk_command_channel_id
        )
        server_config.help_voice_channel_id = (
            input["helpVoiceChannelId"] if "helpVoiceChannelId" in input else server_config.help_voice_channel_id
        )
        server_config.team_channel_id = (
            input["teamChannelId"] if "teamChannelId" in input else server_config.team_channel_id
        )
        server_config.login_message_channel_id = (
            input["loginMessageChannelId"]
            if "loginMessageChannelId" in input
            else server_config.login_message_channel_id
        )
        server_config.default_role_id = (
            input["defaultRoleId"] if "defaultRoleId" in input else server_config.default_role_id
        )
        server_config.feature_flags = (
            dict(zip([x["key"] for x in input["featureFlags"]], [x["value"] for x in input["featureFlags"]]))
            if "featureFlags" in input
            else server_config.feature_flags
        )
        server_config.afk_channel_ids = (
            List(int).extend([int(x) for x in input["afkChannelIds"]])
            if "afkChannelIds" in input
            else server_config.afk_channel_ids
        )

        team_role_ids = List(ServerTeamRoleIdsConfig)
        if "moderatorRoleIds" in input:
            team_role_ids.extend(
                [
                    ServerTeamRoleIdsConfig(x, TeamMemberTypeEnum.moderator, server_config.server.id)
                    for x in input["moderatorRoleIds"]
                ]
            )
        else:
            team_role_ids.extend(
                server_config.team_role_ids.where(lambda x: x.team_member_type == TeamMemberTypeEnum.moderator)
            )

        if "adminRoleIds" in input:
            team_role_ids.extend(
                [
                    ServerTeamRoleIdsConfig(x, TeamMemberTypeEnum.admin, server_config.server.id)
                    for x in input["adminRoleIds"]
                ]
            )
        else:
            team_role_ids.extend(
                server_config.team_role_ids.where(lambda x: x.team_member_type == TeamMemberTypeEnum.admin)
            )

        self._server_configs.update_server_config(server_config)
        if "afkChannelIds" in input:
            self._update_afk_channel_ids(server_config)

        if "moderatorRoleIds" in input or "adminRoleIds" in input:
            server_config.team_role_ids = team_role_ids
            self._update_team_role_ids(server_config)

        self._db.save_changes()
        return server_config

    def _update_afk_channel_ids(self, new_config: ServerConfig):
        old_config = self._server_configs.get_server_config_by_server(new_config.server.id)
        for channel_id in old_config.afk_channel_ids:
            if channel_id in new_config.afk_channel_ids:
                continue

            self._server_configs.delete_server_afk_channel_config(
                ServerAFKChannelIdsConfig(channel_id, new_config.server.id)
            )

        for channel_id in new_config.afk_channel_ids:
            if channel_id in old_config.afk_channel_ids:
                continue

            self._server_configs.add_server_afk_channel_config(
                ServerAFKChannelIdsConfig(channel_id, new_config.server.id)
            )

    def _update_team_role_ids(self, new_config: ServerConfig):
        old_config = self._server_configs.get_server_config_by_server(new_config.server.id)
        for role_id in old_config.team_role_ids:
            if role_id.role_id in new_config.team_role_ids.select(lambda x: int(x.role_id)):
                continue

            self._server_configs.delete_server_team_role_id_config(role_id)

        for role_id in new_config.team_role_ids:
            guild = self._bot.get_guild(new_config.server.discord_id)
            role = guild.get_role(int(role_id.role_id))
            if role is None:
                raise ValueError(f"Invalid roleId")

        for role_id in new_config.team_role_ids:
            if role_id.role_id in old_config.team_role_ids.select(lambda x: str(x.role_id)):
                continue

            self._server_configs.add_server_team_role_id_config(role_id)

        self._bot.loop.create_task(self._config_service.reload_server_config(new_config.server))
        self._permissions.on_ready()
